@@ -1,6 +1,6 @@
 /*
  * iostat: report CPU and I/O statistics
- * (C) 1998-2017 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1998-2018 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -35,7 +35,6 @@
 
 #include "version.h"
 #include "iostat.h"
-#include "common.h"
 #include "ioconf.h"
 #include "rd_stats.h"
 #include "count.h"
@@ -54,14 +53,16 @@ char *sccsid(void) { return (SCCSID); }
 #endif
 
 struct stats_cpu *st_cpu[2];
-unsigned long long uptime[2]  = {0, 0};
-unsigned long long uptime0[2] = {0, 0};
+unsigned long long uptime_cs[2] = {0, 0};
+unsigned long long tot_jiffies[2] = {0, 0};
 struct io_stats *st_iodev[2];
 struct io_hdr_stats *st_hdr_iodev;
 struct io_dlist *st_dev_list;
 
 /* Last group name entered on the command line */
 char group_name[MAX_NAME_LEN];
+/* Number of decimal places */
+int dplaces_nr = -1;
 
 int iodev_nr = 0;	/* Nb of devices and partitions found. Includes nb of device groups */
 int group_nr = 0;	/* Nb of device groups */
@@ -90,13 +91,15 @@ void usage(char *progname)
 #ifdef DEBUG
 	fprintf(stderr, _("Options are:\n"
 			  "[ -c ] [ -d ] [ -h ] [ -k | -m ] [ -N ] [ -s ] [ -t ] [ -V ] [ -x ] [ -y ] [ -z ]\n"
-			  "[ -j { ID | LABEL | PATH | UUID | ... } ] [ --human ] [ -o JSON ]\n"
+			  "[ -j { ID | LABEL | PATH | UUID | ... } ]\n"
+			  "[ --dec={ 0 | 1 | 2 } ] [ --human ] [ -o JSON ]\n"
 			  "[ [ -H ] -g <group_name> ] [ -p [ <device> [,...] | ALL ] ]\n"
 			  "[ <device> [...] | ALL ] [ --debuginfo ]\n"));
 #else
 	fprintf(stderr, _("Options are:\n"
 			  "[ -c ] [ -d ] [ -h ] [ -k | -m ] [ -N ] [ -s ] [ -t ] [ -V ] [ -x ] [ -y ] [ -z ]\n"
-			  "[ -j { ID | LABEL | PATH | UUID | ... } ] [ --human ] [ -o JSON ]\n"
+			  "[ -j { ID | LABEL | PATH | UUID | ... } ]\n"
+			  "[ --dec={ 0 | 1 | 2 } ] [ --human ] [ -o JSON ]\n"
 			  "[ [ -H ] -g <group_name> ] [ -p [ <device> [,...] | ALL ] ]\n"
 			  "[ <device> [...] | ALL ]\n"));
 #endif
@@ -557,10 +560,10 @@ void read_sysfs_dlist_part_stat(int curr, char *dev_name)
 {
 	DIR *dir;
 	struct dirent *drd;
-	char dfile[MAX_PF_NAME], filename[MAX_PF_NAME];
+	char dfile[MAX_PF_NAME], filename[MAX_PF_NAME + 512];
 
-	snprintf(dfile, MAX_PF_NAME, "%s/%s", SYSFS_BLOCK, dev_name);
-	dfile[MAX_PF_NAME - 1] = '\0';
+	snprintf(dfile, sizeof(dfile), "%s/%s", SYSFS_BLOCK, dev_name);
+	dfile[sizeof(dfile) - 1] = '\0';
 
 	/* Open current device directory in /sys/block */
 	if ((dir = opendir(dfile)) == NULL)
@@ -570,8 +573,8 @@ void read_sysfs_dlist_part_stat(int curr, char *dev_name)
 	while ((drd = readdir(dir)) != NULL) {
 		if (!strcmp(drd->d_name, ".") || !strcmp(drd->d_name, ".."))
 			continue;
-		snprintf(filename, MAX_PF_NAME, "%s/%s/%s", dfile, drd->d_name, S_STAT);
-		filename[MAX_PF_NAME - 1] = '\0';
+		snprintf(filename, sizeof(filename), "%s/%s/%s", dfile, drd->d_name, S_STAT);
+		filename[sizeof(filename) - 1] = '\0';
 
 		/* Read current partition stats */
 		read_sysfs_file_stat(curr, filename, drd->d_name);
@@ -853,17 +856,18 @@ void write_sample_timestamp(int tab, struct tm *rectime)
  *
  * IN:
  * @curr	Index in array for current sample statistics.
- * @itv		Interval of time.
+ * @deltot_jiffies
+ *		Number of jiffies spent on the interval by all processors.
  ***************************************************************************
  */
-void write_plain_cpu_stat(int curr, unsigned long long itv)
+void write_plain_cpu_stat(int curr, unsigned long long deltot_jiffies)
 {
 	printf("avg-cpu:  %%user   %%nice %%system %%iowait  %%steal   %%idle\n");
 
 	printf("       ");
 	cprintf_pc(DISPLAY_UNIT(flags), 6, 7, 2,
-		   ll_sp_value(st_cpu[!curr]->cpu_user,   st_cpu[curr]->cpu_user,   itv),
-		   ll_sp_value(st_cpu[!curr]->cpu_nice,   st_cpu[curr]->cpu_nice,   itv),
+		   ll_sp_value(st_cpu[!curr]->cpu_user, st_cpu[curr]->cpu_user, deltot_jiffies),
+		   ll_sp_value(st_cpu[!curr]->cpu_nice, st_cpu[curr]->cpu_nice, deltot_jiffies),
 		   /*
 		    * Time spent in system mode also includes time spent servicing
 		    * hard and soft interrupts.
@@ -871,12 +875,12 @@ void write_plain_cpu_stat(int curr, unsigned long long itv)
 		   ll_sp_value(st_cpu[!curr]->cpu_sys + st_cpu[!curr]->cpu_softirq +
 			       st_cpu[!curr]->cpu_hardirq,
 			       st_cpu[curr]->cpu_sys + st_cpu[curr]->cpu_softirq +
-			       st_cpu[curr]->cpu_hardirq, itv),
-		   ll_sp_value(st_cpu[!curr]->cpu_iowait, st_cpu[curr]->cpu_iowait, itv),
-		   ll_sp_value(st_cpu[!curr]->cpu_steal,  st_cpu[curr]->cpu_steal,  itv),
+			       st_cpu[curr]->cpu_hardirq, deltot_jiffies),
+		   ll_sp_value(st_cpu[!curr]->cpu_iowait, st_cpu[curr]->cpu_iowait, deltot_jiffies),
+		   ll_sp_value(st_cpu[!curr]->cpu_steal, st_cpu[curr]->cpu_steal, deltot_jiffies),
 		   (st_cpu[curr]->cpu_idle < st_cpu[!curr]->cpu_idle) ?
 		   0.0 :
-		   ll_sp_value(st_cpu[!curr]->cpu_idle,   st_cpu[curr]->cpu_idle,   itv));
+		   ll_sp_value(st_cpu[!curr]->cpu_idle, st_cpu[curr]->cpu_idle, deltot_jiffies));
 
 	printf("\n\n");
 }
@@ -888,15 +892,16 @@ void write_plain_cpu_stat(int curr, unsigned long long itv)
  * IN:
  * @tab		Number of tabs to print.
  * @curr	Index in array for current sample statistics.
- * @itv		Interval of time.
+ * @deltot_jiffies
+ *		Number of jiffies spent on the interval by all processors.
  ***************************************************************************
  */
-void write_json_cpu_stat(int tab, int curr, unsigned long long itv)
+void write_json_cpu_stat(int tab, int curr, unsigned long long deltot_jiffies)
 {
 	xprintf0(tab, "\"avg-cpu\":  {\"user\": %.2f, \"nice\": %.2f, \"system\": %.2f,"
 		      " \"iowait\": %.2f, \"steal\": %.2f, \"idle\": %.2f}",
-		 ll_sp_value(st_cpu[!curr]->cpu_user,   st_cpu[curr]->cpu_user,   itv),
-		 ll_sp_value(st_cpu[!curr]->cpu_nice,   st_cpu[curr]->cpu_nice,   itv),
+		 ll_sp_value(st_cpu[!curr]->cpu_user, st_cpu[curr]->cpu_user, deltot_jiffies),
+		 ll_sp_value(st_cpu[!curr]->cpu_nice, st_cpu[curr]->cpu_nice, deltot_jiffies),
 		 /*
 		  * Time spent in system mode also includes time spent servicing
 		  * hard and soft interrupts.
@@ -904,12 +909,12 @@ void write_json_cpu_stat(int tab, int curr, unsigned long long itv)
 		 ll_sp_value(st_cpu[!curr]->cpu_sys + st_cpu[!curr]->cpu_softirq +
 			     st_cpu[!curr]->cpu_hardirq,
 			     st_cpu[curr]->cpu_sys + st_cpu[curr]->cpu_softirq +
-			     st_cpu[curr]->cpu_hardirq, itv),
-		 ll_sp_value(st_cpu[!curr]->cpu_iowait, st_cpu[curr]->cpu_iowait, itv),
-		 ll_sp_value(st_cpu[!curr]->cpu_steal,  st_cpu[curr]->cpu_steal,  itv),
+			     st_cpu[curr]->cpu_hardirq, deltot_jiffies),
+		 ll_sp_value(st_cpu[!curr]->cpu_iowait, st_cpu[curr]->cpu_iowait, deltot_jiffies),
+		 ll_sp_value(st_cpu[!curr]->cpu_steal, st_cpu[curr]->cpu_steal, deltot_jiffies),
 		 (st_cpu[curr]->cpu_idle < st_cpu[!curr]->cpu_idle) ?
 		 0.0 :
-		 ll_sp_value(st_cpu[!curr]->cpu_idle,   st_cpu[curr]->cpu_idle,   itv));
+		 ll_sp_value(st_cpu[!curr]->cpu_idle, st_cpu[curr]->cpu_idle, deltot_jiffies));
 }
 
 /*
@@ -918,17 +923,52 @@ void write_json_cpu_stat(int tab, int curr, unsigned long long itv)
  *
  * IN:
  * @curr	Index in array for current sample statistics.
- * @itv		Interval of time.
  * @tab		Number of tabs to print (JSON format only).
  ***************************************************************************
  */
-void write_cpu_stat(int curr, unsigned long long itv, int tab)
+void write_cpu_stat(int curr, int tab)
 {
+	unsigned long long deltot_jiffies;
+
+	/*
+	 * Compute the total number of jiffies spent by all processors.
+	 * NB: Don't add cpu_guest/cpu_guest_nice because cpu_user/cpu_nice
+	 * already include them.
+	 */
+	tot_jiffies[curr] = st_cpu[curr]->cpu_user + st_cpu[curr]->cpu_nice +
+			    st_cpu[curr]->cpu_sys + st_cpu[curr]->cpu_idle +
+			    st_cpu[curr]->cpu_iowait + st_cpu[curr]->cpu_hardirq +
+			    st_cpu[curr]->cpu_steal + st_cpu[curr]->cpu_softirq;
+
+	/* Total number of jiffies spent on the interval */
+	deltot_jiffies = get_interval(tot_jiffies[!curr], tot_jiffies[curr]);
+
+#ifdef DEBUG
+		if (DISPLAY_DEBUG(flags)) {
+			/* Debug output */
+			fprintf(stderr, "deltot_jiffies=%llu st_cpu[curr]{ cpu_user=%llu cpu_nice=%llu "
+					"cpu_sys=%llu cpu_idle=%llu cpu_iowait=%llu cpu_steal=%llu "
+					"cpu_hardirq=%llu cpu_softirq=%llu cpu_guest=%llu "
+					"cpu_guest_nice=%llu }\n",
+				deltot_jiffies,
+				st_cpu[curr]->cpu_user,
+				st_cpu[curr]->cpu_nice,
+				st_cpu[curr]->cpu_sys,
+				st_cpu[curr]->cpu_idle,
+				st_cpu[curr]->cpu_iowait,
+				st_cpu[curr]->cpu_steal,
+				st_cpu[curr]->cpu_hardirq,
+				st_cpu[curr]->cpu_softirq,
+				st_cpu[curr]->cpu_guest,
+				st_cpu[curr]->cpu_guest_nice);
+		}
+#endif
+
 	if (DISPLAY_JSON_OUTPUT(flags)) {
-		write_json_cpu_stat(tab, curr, itv);
+		write_json_cpu_stat(tab, curr, deltot_jiffies);
 	}
 	else {
-		write_plain_cpu_stat(curr, itv);
+		write_plain_cpu_stat(curr, deltot_jiffies);
 	}
 }
 
@@ -955,9 +995,11 @@ void write_disk_stat_header(int *fctr, int *tab)
 		return;
 	}
 
+	if (!DISPLAY_HUMAN_READ(flags)) {
+		printf("Device       ");
+	}
 	if (DISPLAY_EXTENDED(flags)) {
 		/* Extended stats */
-		printf("Device       ");
 		if (DISPLAY_SHORT_OUTPUT(flags)) {
 			printf("      tps");
 			if (DISPLAY_MEGABYTES(flags)) {
@@ -969,7 +1011,7 @@ void write_disk_stat_header(int *fctr, int *tab)
 			else {
 				printf("     sec/s");
 			}
-			printf("    rqm/s   await aqu-sz  areq-sz  %%util\n");
+			printf("    rqm/s   await aqu-sz  areq-sz  %%util");
 		}
 		else {
 			printf("     r/s     w/s");
@@ -983,22 +1025,26 @@ void write_disk_stat_header(int *fctr, int *tab)
 				printf("    rsec/s    wsec/s");
 			}
 			printf("   rrqm/s   wrqm/s  %%rrqm  %%wrqm r_await w_await"
-			       " aqu-sz rareq-sz wareq-sz  svctm  %%util\n");
+			       " aqu-sz rareq-sz wareq-sz  svctm  %%util");
 		}
 	}
 	else {
 		/* Basic stats */
-		printf("Device             tps");
+		printf("      tps");
 		if (DISPLAY_KILOBYTES(flags)) {
-			printf("    kB_read/s    kB_wrtn/s    kB_read    kB_wrtn\n");
+			printf("    kB_read/s    kB_wrtn/s    kB_read    kB_wrtn");
 		}
 		else if (DISPLAY_MEGABYTES(flags)) {
-			printf("    MB_read/s    MB_wrtn/s    MB_read    MB_wrtn\n");
+			printf("    MB_read/s    MB_wrtn/s    MB_read    MB_wrtn");
 		}
 		else {
-			printf("   Blk_read/s   Blk_wrtn/s   Blk_read   Blk_wrtn\n");
+			printf("   Blk_read/s   Blk_wrtn/s   Blk_read   Blk_wrtn");
 		}
 	}
+	if (DISPLAY_HUMAN_READ(flags)) {
+		printf(" Device");
+	}
+	printf("\n");
 }
 
 /*
@@ -1022,11 +1068,7 @@ void write_plain_ext_stat(unsigned long long itv, int fctr,
 			  struct io_stats *ioj, char *devname, struct ext_disk_stats *xds,
 			  struct ext_io_stats *xios)
 {
-	if (DISPLAY_HUMAN_READ(flags)) {
-		cprintf_in(IS_STR, "%s\n", devname, 0);
-		printf("%13s", "");
-	}
-	else {
+	if (!DISPLAY_HUMAN_READ(flags)) {
 		cprintf_in(IS_STR, "%-13s", devname, 0);
 	}
 
@@ -1061,7 +1103,6 @@ void write_plain_ext_stat(unsigned long long itv, int fctr,
 		cprintf_pc(DISPLAY_UNIT(flags), 1, 6, 2,
 			   shi->used ? xds->util / 10.0 / (double) shi->used
 				     : xds->util / 10.0);	/* shi->used should never be zero here */
-		printf("\n");
 	}
 	else {
 		/* r/s  w/s */
@@ -1102,8 +1143,12 @@ void write_plain_ext_stat(unsigned long long itv, int fctr,
 		cprintf_pc(DISPLAY_UNIT(flags), 1, 6, 2,
 			   shi->used ? xds->util / 10.0 / (double) shi->used
 				     : xds->util / 10.0);	/* shi->used should never be zero here */
-		printf("\n");
 	}
+
+	if (DISPLAY_HUMAN_READ(flags)) {
+		cprintf_in(IS_STR, " %s", devname, 0);
+	}
+	printf("\n");
 }
 
 /*
@@ -1308,11 +1353,7 @@ void write_plain_basic_stat(unsigned long long itv, int fctr,
 {
 	double rsectors, wsectors;
 
-	if (DISPLAY_HUMAN_READ(flags)) {
-		cprintf_in(IS_STR, "%s\n", devname, 0);
-		printf("%13s", "");
-	}
-	else {
+	if (!DISPLAY_HUMAN_READ(flags)) {
 		cprintf_in(IS_STR, "%-13s", devname, 0);
 	}
 	cprintf_f(NO_UNIT, 1, 8, 2,
@@ -1330,6 +1371,9 @@ void write_plain_basic_stat(unsigned long long itv, int fctr,
 					: (unsigned long long) rd_sec / fctr,
 		    DISPLAY_UNIT(flags) ? (unsigned long long) wr_sec
 					: (unsigned long long) wr_sec / fctr);
+	if (DISPLAY_HUMAN_READ(flags)) {
+		cprintf_in(IS_STR, " %s", devname, 0);
+	}
 	printf("\n");
 }
 
@@ -1461,33 +1505,9 @@ void write_stats(int curr, struct tm *rectime)
 #endif
 	}
 
-	/* Interval is multiplied by the number of processors */
-	itv = get_interval(uptime[!curr], uptime[curr]);
-
 	if (DISPLAY_CPU(flags)) {
-#ifdef DEBUG
-		if (DISPLAY_DEBUG(flags)) {
-			/* Debug output */
-			fprintf(stderr, "itv=%llu st_cpu[curr]{ cpu_user=%llu cpu_nice=%llu "
-					"cpu_sys=%llu cpu_idle=%llu cpu_iowait=%llu cpu_steal=%llu "
-					"cpu_hardirq=%llu cpu_softirq=%llu cpu_guest=%llu "
-					"cpu_guest_nice=%llu }\n",
-				itv,
-				st_cpu[curr]->cpu_user,
-				st_cpu[curr]->cpu_nice,
-				st_cpu[curr]->cpu_sys,
-				st_cpu[curr]->cpu_idle,
-				st_cpu[curr]->cpu_iowait,
-				st_cpu[curr]->cpu_steal,
-				st_cpu[curr]->cpu_hardirq,
-				st_cpu[curr]->cpu_softirq,
-				st_cpu[curr]->cpu_guest,
-				st_cpu[curr]->cpu_guest_nice);
-		}
-#endif
-
 		/* Display CPU utilization */
-		write_cpu_stat(curr, itv, tab);
+		write_cpu_stat(curr, tab);
 
 		if (DISPLAY_JSON_OUTPUT(flags)) {
 			if (DISPLAY_DISK(flags)) {
@@ -1497,10 +1517,8 @@ void write_stats(int curr, struct tm *rectime)
 		}
 	}
 
-	if (cpu_nr > 1) {
-		/* On SMP machines, reduce itv to one processor (see note above) */
-		itv = get_interval(uptime0[!curr], uptime0[curr]);
-	}
+	/* Calculate time interval in 1/100th of a second */
+	itv = get_interval(uptime_cs[!curr], uptime_cs[curr]);
 
 	if (DISPLAY_DISK(flags)) {
 		struct io_stats *ioi, *ioj;
@@ -1624,22 +1642,11 @@ void rw_io_stat_loop(long int count, struct tm *rectime)
 	setbuf(stdout, NULL);
 
 	do {
-		if (cpu_nr > 1) {
-			/*
-			 * Read system uptime (only for SMP machines).
-			 * Init uptime0. So if /proc/uptime cannot fill it,
-			 * this will be done by /proc/stat.
-			 */
-			uptime0[curr] = 0;
-			read_uptime(&(uptime0[curr]));
-		}
+		/* Read system uptime (only for SMP machines) */
+		read_uptime(&(uptime_cs[curr]));
 
-		/*
-		 * Read stats for CPU "all" and 0.
-		 * Note that stats for CPU 0 are not used per se. It only makes
-		 * read_stat_cpu() fill uptime0.
-		 */
-		read_stat_cpu(st_cpu[curr], 2, &(uptime[curr]), &(uptime0[curr]));
+		/* Read stats for CPU "all" */
+		read_stat_cpu(st_cpu[curr], 1);
 
 		if (dlist_idx) {
 			/*
@@ -1730,9 +1737,6 @@ int main(int argc, char **argv)
 	/* Init color strings */
 	init_colors();
 
-	/* Get HZ */
-	get_HZ();
-
 	/* Allocate structures for device list */
 	if (argc > 1) {
 		salloc_dev_list(argc - 1 + count_csvalues(argc, argv));
@@ -1804,6 +1808,15 @@ int main(int argc, char **argv)
 			opt++;
 		}
 
+		else if (!strncmp(argv[opt], "--dec=", 6) && (strlen(argv[opt]) == 7)) {
+			/* Get number of decimal places */
+			dplaces_nr = atoi(argv[opt] + 6);
+			if ((dplaces_nr < 0) || (dplaces_nr > 2)) {
+				usage(argv[0]);
+			}
+			opt++;
+		}
+
 		else if (!strcmp(argv[opt], "-j")) {
 			if (!argv[++opt]) {
 				usage(argv[0]);
